"""
Helper functions for URLs.
"""

# Standard Library
from urllib.parse import urljoin

# Django
from django.conf import settings
from django.urls import reverse


def reverse_absolute(viewname: str, args: list | None = None) -> str:
    """
    Return absolute URL for a view name.

    Similar to Django's ``reverse()``, but returns an absolute URL.
    """

    return urljoin(base=settings.SITE_URL, url=reverse(viewname=viewname, args=args))



"""
App config
"""

# Django
from django.apps import AppConfig
from django.utils.text import format_lazy

# AA Time Zones
from timezones import __title_translated__, __version__


class AaTimezonesConfig(AppConfig):
    """
    AaTimezonesConfig
    """

    name: str = "timezones"
    label: str = "timezones"
    verbose_name = format_lazy(
        "{app_title} v{version}", app_title=__title_translated__, version=__version__
    )



"""
App urls config
"""

# Django
from django.urls import path

# AA Time Zones
from timezones import views

app_name: str = "timezones"  # pylint: disable=invalid-name

urlpatterns = [
    path("", views.index, name="index"),
    path("<str:timestamp>/", views.index, name="index"),
]



"""
App settings and checks
"""

# Django
from django.apps import apps
from django.conf import settings


def allianceauth_discordbot_installed():
    """
    Check if allianceauth-discordbot is installed and active

    :return:
    """

    return apps.is_installed("aadiscordbot")


def debug_enabled() -> bool:
    """
    Check if DEBUG is enabled

    :return:
    :rtype:
    """

    return settings.DEBUG



"""
AppLogger provider
"""

# Standard Library
import logging

# AA Time Zones
from timezones import __title__


class AppLogger(logging.LoggerAdapter):
    """
    Custom logger adapter that adds a prefix to log messages.

    Taken from the `allianceauth-app-utils` package.
    Credits to: Erik Kalkoken
    """

    def __init__(self, my_logger: logging.Logger):
        """
        Initializes the AppLogger with a logger and a prefix.

        :param my_logger: Logger instance
        :type my_logger: logging.Logger
        """

        super().__init__(my_logger, {})

        self.prefix = __title__

    def process(self, msg, kwargs):
        """
        Prepares the log message by adding the prefix.

        :param msg: Log message
        :type msg: str
        :param kwargs: Additional keyword arguments
        :type kwargs: dict
        :return: Prefixed log message and kwargs
        :rtype: tuple
        """

        return f"[{self.prefix}] {msg}", kwargs



"""
Admin backend config
"""

# Django
from django.contrib import admin, messages
from django.db import transaction
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext

# AA Time Zones
from timezones.models import Timezones


@admin.register(Timezones)
class TimezonesAdmin(admin.ModelAdmin):
    """
    timezones admin
    Configure timezones that should be displayed
    """

    list_display = ("_panel_name", "_timezone", "is_enabled")
    ordering = ("panel_name",)
    list_filter = ("is_enabled",)

    @admin.display(description=_("Panel name"), ordering="panel_name")
    def _panel_name(self, obj):
        return obj.panel_name

    @admin.display(description=_("Time zone"), ordering="timezone__timezone_name")
    def _timezone(self, obj):
        return obj.timezone.timezone_name

    actions = ("mark_as_active", "mark_as_inactive")

    @admin.action(description=_("Activate selected timezones"))
    def mark_as_active(self, request, queryset):
        """
        Mark timezone as active
        :param request:
        :param queryset:
        """

        notifications_count = 0
        failed = 0

        for obj in queryset:
            try:
                with transaction.atomic():
                    obj.is_enabled = True
                    obj.save()

                    notifications_count += 1
            except Exception:  # pylint: disable=broad-exception-caught
                failed += 1

        messages.success(
            request,
            ngettext(
                singular="Activated {notifications_count} timezone",
                plural="Activated {notifications_count} timezones",
                number=notifications_count,
            ).format(notifications_count=notifications_count),
        )

        if failed:
            messages.error(
                request,
                ngettext(
                    singular="Failed to activate {failed} timezone",
                    plural="Failed to activate {failed} timezones",
                    number=failed,
                ).format(failed=failed),
            )

        if queryset.count() - failed > 0:
            messages.success(
                request,
                ngettext(
                    singular="Activated {notifications_count} timezone",
                    plural="Activated {notifications_count} timezones",
                    number=notifications_count,
                ).format(notifications_count=notifications_count),
            )

    @admin.action(description=_("Deactivate selected timezones"))
    def mark_as_inactive(self, request, queryset):
        """
        Mark timezone as inactive
        :param request:
        :param queryset:
        """

        notifications_count = 0
        failed = 0

        for obj in queryset:
            try:
                with transaction.atomic():
                    obj.is_enabled = False
                    obj.save()

                    notifications_count += 1
            except Exception:  # pylint: disable=broad-exception-caught
                failed += 1

        if failed:
            messages.error(
                request,
                ngettext(
                    singular="Failed to deactivate {failed} timezone",
                    plural="Failed to deactivate {failed} timezones",
                    number=failed,
                ).format(failed=failed),
            )

        if queryset.count() - failed > 0:
            messages.success(
                request,
                ngettext(
                    singular="Deactivated {notifications_count} timezone",
                    plural="Deactivated {notifications_count} timezones",
                    number=notifications_count,
                ).format(notifications_count=notifications_count),
            )



# Generated by Django 4.0.10 on 2023-08-07 14:35

# Django
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("timezones", "0003_auto_20201003_1748"),
    ]

    operations = [
        migrations.DeleteModel(
            name="AaTimezones",
        ),
    ]



"""
Tests for the auth_hooks module.
"""

# Standard Library
from unittest import TestCase
from unittest.mock import patch

# Django
from django.test import RequestFactory, modify_settings

# Alliance Auth
from allianceauth.services.hooks import MenuItemHook

# AA Time Zones
from timezones.auth_hooks import AaTimezonesMenuItem, register_menu, register_urls


class TestAaTimezonesMenuItem(TestCase):
    """
    Tests for the AaTimezonesMenuItem class
    """

    def setUp(self):
        """
        Set up

        :return:
        :rtype:
        """

        self.request = RequestFactory().get("/timezones")
        self.menu_item = AaTimezonesMenuItem()

    def test_render_calls_super(self):
        """
        Test render calls super

        :return:
        :rtype:
        """

        with patch.object(MenuItemHook, "render") as mock_super:
            self.menu_item.render(self.request)
            mock_super.assert_called_once_with(self.menu_item, self.request)


class TestRegisterMenu(TestCase):
    """
    Tests for the register_menu function
    """

    def test_returns_instance_of_AaTimezonesMenuItem(self):
        """
        Test returns instance of AaTimezonesMenuItem

        :return:
        :rtype:
        """

        result = register_menu()
        self.assertIsInstance(result, AaTimezonesMenuItem)


class TestRegisterUrls(TestCase):
    """
    Tests for the register_urls function
    """

    def test_returns_UrlHook_with_correct_attributes(self):
        """
        Test returns UrlHook with correct attributes

        :return:
        :rtype:
        """

        result = register_urls()

        self.assertEqual(result.__class__.__name__, "UrlHook")
        self.assertEqual(result.include_pattern.pattern._regex, r"^timezones/")
        self.assertEqual(result.excluded_views, {"timezones.views.index"})


class TestRegisterCogs(TestCase):
    """
    Tests for the register_cogs function
    """

    @modify_settings(INSTALLED_APPS={"append": "aadiscordbot"})
    def test_register_cogs_true(self):
        """
        Test register_cogs returns the correct cogs when aadiscordbot is installed

        :return:
        :rtype:
        """

        # AA Time Zones
        from timezones.auth_hooks import register_cogs

        cogs = register_cogs()

        self.assertEqual(cogs, ["timezones.aadiscordbot.cogs.time"])



"""
Unit tests for the timezones.helper.urls helper.
"""

# Standard Library
from unittest.mock import patch

# AA Time Zones
from timezones.helper.urls import reverse_absolute
from timezones.tests import BaseTestCase


class TestReverseAbsolute(BaseTestCase):
    """
    Test reverse_absolute helper
    """

    @patch("timezones.helper.urls.reverse")
    def test_returns_absolute_url_for_valid_viewname(self, mock_reverse):
        """
        Test returns absolute URL for valid viewname

        :param mock_reverse:
        :type mock_reverse:
        :return:
        :rtype:
        """

        mock_reverse.return_value = "/mocked-path/"

        with self.settings(SITE_URL="https://example.com"):
            result = reverse_absolute("mock_view")

        self.assertEqual(result, "https://example.com/mocked-path/")

    @patch("timezones.helper.urls.reverse")
    def test_handles_none_args_correctly(self, mock_reverse):
        """
        Test handles None args correctly

        :param mock_reverse:
        :type mock_reverse:
        :return:
        :rtype:
        """

        mock_reverse.return_value = "/mocked-path/"

        with self.settings(SITE_URL="https://example.com"):
            result = reverse_absolute("mock_view", args=None)

        self.assertEqual(result, "https://example.com/mocked-path/")

    @patch("timezones.helper.urls.reverse")
    def test_includes_args_in_generated_url(self, mock_reverse):
        """
        Test includes args in generated URL

        :param mock_reverse:
        :type mock_reverse:
        :return:
        :rtype:
        """

        mock_reverse.return_value = "/mocked-path/123/"

        with self.settings(SITE_URL="https://example.com"):
            result = reverse_absolute("mock_view", args=[123])

        self.assertEqual(result, "https://example.com/mocked-path/123/")
